/* The main window instance */

const { ipcRenderer } = require('electron'),
    logger = require('../common/logger.js'),
    ejs = require('ejs'),
    e = require('express');

/* Global variables */
var countKaraInQueue = 0;
var currentSong = 0;
var dragCounter = 0;
var leavedElement;
var isDnDFromDB = false;

function updatePlayPauseButton(state) {
    logger.debug('instance', `State was ${state}`);
    if (state === 'play') {
        /* Play */
        $('#commandPlay').children().addClass('fa-pause');
        $('#commandPlay').children().removeClass('fa-play');
    } else {
        /* Pause or Stop */
        $('#commandPlay').children().addClass('fa-play');
        $('#commandPlay').children().removeClass('fa-pause');
    }
}

function addIpcToButton(btnId, ipc) {
    document.getElementById(btnId).addEventListener('click', () => {
        ipc.forEach(ipc => {
            ipcRenderer.send(ipc);
        });
    });
}

function autoFillSettings() {
    $('#inputHost').val(config.content.lektord.host);
    $('#inputPort').val(config.content.lektord.port);
    $('#inputAttachMode').prop('checked', true);
    $('#inputDbPath').val(config.content.database.path);
    $('#inputLogLektord').prop('checked', config.content.log.lektord);
    $(`#log${config.content.log.level.toUpperCase()}`).prop('checked', true);
    $('#inputKurisuUrl').val(config.content.kurisu.url);
    $('#inputClientPort').val(config.content.client.port);
}

function flushFillSettings() {
    const set = {
        kurisuurl: $('#inputKurisuUrl').val(),
        clientport: $('#inputClientPort').val(),
        host: $('#inputHost').val(),
        port: $('#inputPort').val(),
        attachmode: $('#inputAttachMode').prop('checked'),
        db: $('#inputDbPath').val(),
        loglektord: $('#inputLogLektord').prop('checked'),
        loglevel: {
            ERROR: $('#logERROR').prop('checked'),
            WARNING: $('#logWARNING').prop('checked'),
            INFO: $('#logINFO').prop('checked'),
            DEBUG: $('#logDEBUG').prop('checked'),
        },
    };
    ipcRenderer.send('set-settings', set);
}

window.onload = () => {
    /* Set things up for the settings modal*/
    var modal = document.getElementById('modal-settings');
    var btn = document.getElementById('btn-settings');
    var span = document.getElementsByClassName('modal-close')[0];

    btn.onclick = function () {
        modal.style.display = 'block';
    };

    span.onclick = function () {
        modal.style.display = 'none';
        flushFillSettings();
    };

    window.onclick = function (event) {
        if (event.target == modal) {
            modal.style.display = 'none';
        }
    };

    /* Add IPC to button list */
    addIpcToButton('closeButton', ['close-app']);

    addIpcToButton('commandPlay', ['cmd-play']);
    addIpcToButton('commandStop', ['cmd-stop']);
    addIpcToButton('commandClear', ['cmd-clear']);

    addIpcToButton('selectAdvence', ['select-advence']);
    addIpcToButton('selectDatabase', ['select-database', 'reload-db-request']);
    addIpcToButton('selectPlaylist', ['select-playlist']);
    addIpcToButton('selectPool', ['select-pool']);
    addIpcToButton('openMdtView', ['select-mdt']);

    addIpcToButton('openKurisu', ['toggle-kurisu']);
    addIpcToButton('openUserView', ['toggle-client-view']);

    ipcRenderer.send('verify-lektord');
    logger.debug('instance', 'Window loaded');

    $('#filterInput').on('keypress', e => {
        /* On 'Return'. */
        if (e.which != 13) return;
        logger.debug('instance', `Send filter for: ${$('#filterInput').val()}`);
        ipcRenderer.send('reload-db-request', {
            search: `${$('#filterInput').val()}`,
        });
    });

    document.addEventListener(
        'keydown',
        e => {
            if (e.ctrlKey && e.keyCode == 70) {
                /* ctrl + F */
                logger.debug('instance', 'Focus find bar');
                $('#filterInput').focus();
            } else if (e.ctrlKey && e.keyCode == 67) {
                /* ctrl + C */
                logger.debug('instance', `Focus current kara in queue, id is "#pos-${currentSong + 1}"`);
                $('#panelRight').animate(
                    {
                        scrollTop: 0,
                    },
                    0
                );
                var offset = $(window).height() / 2;
                $('#panelRight').animate(
                    {
                        scrollTop:
                            $(`#pos-${currentSong + 1}`)
                                .parent()
                                .offset().top - offset,
                    },
                    0
                );
            }
        },
        false
    );

    /* Autofill settings */
    autoFillSettings();
};

/* Create the left panel */
ipcRenderer.on('reload-db-responce', (event, arg) => {
    logger.debug('instance', `Web page got reload-db`);
    document.getElementById('panelLeft').innerHTML = '';
    arg.forEach(kara => {
        ejs.renderFile(__dirname + '/views/karaDBListItem.ejs', { kara: kara }, (err, data) => {
            if (err) logger.error('instance', err);
            $('#panelLeft').append(data);
        });
    });
    [].forEach.call(document.querySelectorAll('#panelLeft .karaCard'), addDBKaraEventHandlers);
});

/* Create the right panel: the queue */
ipcRenderer.on('reload-queue-responce', (event, arg) => {
    logger.debug('instance', `Web page got reload-queue`);
    countKaraInQueue = 0;
    document.getElementById('panelRight').innerHTML = '';
    arg.forEach(kara => {
        countKaraInQueue++;
        ejs.renderFile(__dirname + '/views/karaQueueListItem.ejs', { kara: kara }, (err, data) => {
            if (err) logger.error('instance', err);
            $('#panelRight').append(data);
        });
    });
    [].forEach.call(document.querySelectorAll('#panelRight .karaCard'), addQueueKaraEventHandlers);
});

ipcRenderer.on('send-state', (event, state) => updatePlayPauseButton(state));

ipcRenderer.on('send-song-time-data', (event, timeData) => {
    document.getElementById('progressBar').style.width = `${
        (timeData.elapsed / timeData.total) * document.documentElement.clientWidth
    }px`;
    if (timeData.state == 'play' || timeData.state == 'pause') {
        if (currentSong != timeData.song) {
            document.getElementsByClassName('karaQueue')[currentSong].style.background = '#4e5d6c';
            currentSong = timeData.song;
        }
        document.getElementsByClassName('karaQueue')[timeData.song].style.background = '#6b7d8e';
    } else if (document.getElementsByClassName('karaQueue')[timeData.song]) {
        document.getElementsByClassName('karaQueue')[timeData.song].style.background = '#4e5d6c';
    }
});

ipcRenderer.on('send-runnings', (event, arg) => {
    $('#running-lektord').prop('checked', arg.lektord);
    $('#running-klkt').prop('checked', arg.klkt);
});

ipcRenderer.on('send-lektord-is-attached', (event, arg) => {
    logger.debug('instance', `Lektord is in attached mode? ${arg}`);
});

/*
 * Drag'n'drop functions
 */

function addDBKaraEventHandlers(element) {
    element.addEventListener('dragstart', onDragStartDB, false);
    element.addEventListener('dragend', onDragEnd, false);
    element.getElementsByClassName('karaAddBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('add-kara-queue-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        })
    );
    element.getElementsByClassName('karaInsertBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('insert-kara-queue-id', {
            id: element.getElementsByClassName('karaID')[0].innerText,
        })
    );
}

function addQueueKaraEventHandlers(element) {
    element.addEventListener('dragstart', onDragStartQueue, false);
    element.addEventListener('dragenter', onDragEnter, false);
    element.addEventListener('dragover', onDragOver, false);
    element.addEventListener('dragleave', ondragLeave, false);
    element.addEventListener('drop', onDrop, false);
    element.addEventListener('dragend', onDragEnd, false);
    element.getElementsByClassName('karaDeleteBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('delete-kara-queue-pos', {
            position: element.getElementsByClassName('karaPosition')[0].innerText,
        })
    );
    element.getElementsByClassName('karaPlayBtn')[0].addEventListener('click', () =>
        ipcRenderer.send('play-kara-queue-pos', {
            position: element.getElementsByClassName('karaPosition')[0].innerText,
        })
    );
}

function onDragStartDB(event) {
    isDnDFromDB = true;
    event.dataTransfer.effectAllowed = 'move';
    event.currentTarget.style.opacity = 0.4;
    event.dataTransfer.setData('text/plain', event.currentTarget.getElementsByClassName('karaID')[0].innerText);
}

function onDragStartQueue(event) {
    event.dataTransfer.effectAllowed = 'move';
    event.currentTarget.style.opacity = 0.4;
    event.dataTransfer.setData('text/plain', event.currentTarget.getElementsByClassName('karaPosition')[0].innerText);
}

function onDragEnter(event) {
    if (event.currentTarget) {
        if (leavedElement != event.currentTarget && leavedElement) {
            leavedElement.style.borderTop = ''; // style is undefined !
        }
        event.currentTarget.style.borderTop = '3px solid grey';
        dragCounter++;
    }
}

function onDragOver(event) {
    if (event.preventDefault) {
        event.preventDefault();
    }
    event.dataTransfer.dropEffect = 'move';
}

function ondragLeave(event) {
    dragCounter--;
    leavedElement = event.currentTarget;
    if (dragCounter == 0) {
        event.currentTarget.style.borderTop = '';
    }
}

function onDrop(event) {
    event.currentTarget.style.borderTop = '';
    if (isDnDFromDB) {
        ipcRenderer.send('add-kara-queue-pos', {
            id: event.dataTransfer.getData('text/plain'),
            position: event.currentTarget.getElementsByClassName('karaPosition')[0].innerText,
            queueSize: countKaraInQueue,
        });
    } else {
        ipcRenderer.send('queue-moved-kara', {
            from: event.dataTransfer.getData('text/plain'),
            to: event.currentTarget.getElementsByClassName('karaPosition')[0].innerText,
        });
    }
    isDnDFromDB = false;
    return false;
}

function onDragEnd(event) {
    event.currentTarget.style.opacity = 1;
    dragCounter = 0;
}
